package jp.co.sds_corp.syslog.viewer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import jp.co.sds_corp.syslog.ISyslogListener;
import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.syslog.SyslogService;
import jp.co.sds_corp.syslog.filter.FilterUtility;
import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.syslog.viewer.statistics.StatisticsDialog;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.DOMWriter;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.manager.DaemonManager;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.syslog.SyslogMsg;
import jp.co.sds_corp.util.window.ExtFileFilter;
import jp.co.sds_corp.util.window.FontChooser;
import jp.co.sds_corp.util.window.StatusBar;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* SyslogViewerのメインパネルです。
*/
public class SyslogPanel extends JPanel
	implements IConfigurable, ISyslogListener
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	SyslogService	service;
	SyslogParam	param;

	Component parent;
	JScrollPane sp;
	SyslogModel	model;
	SyslogTable	table;
	StatusBar	status;

	AboutDialog	about;
	DescDialog	desc;
	SetupDialog	setup;
	SearchDialog	search;
	StatisticsDialog	stat;
	FontChooser	font;
	Vector<SyslogEvent>	vecLog = new Vector<SyslogEvent>();
	int	nRecv;
	ResourceBundle res;

	public SyslogPanel() {
		super();
	}

	public void init( IConfiguration config ) throws Exception {
		IConfiguration c = config.getChild( "Viewer" );

		res = ResourceBundle.getBundle( "resource.viewer" );
		param = new SyslogParam();
		param.init( c );
		setFont( param.getFont() );
		setBounds( param.getBounds() );

		setLayout( new BorderLayout() );
		table = createTable( param );
		status = createStatusBar();

		desc = createDescDialog( param );
		setup = createSetupDialog( param );
		search = createSearchDialog( param );
		stat = createStatisticsDialog( param );
		font = new FontChooser();
		table.getTableHeader().setFont( param.getFont() );
		table.setFont( param.getFont() );
		parent = SwingUtilities.getRoot( this );
		if ( parent instanceof Dialog ) {
			about = new AboutDialog( (Dialog )parent );
		} else if ( parent instanceof Frame ) {
			about = new AboutDialog( (Frame )parent );
		} else {
			about = new AboutDialog();
		}
		param.getPopup().setParent( this );

		service = SyslogService.getInstance();
		service.addSyslogListener( this );
	}

	protected SyslogTable createTable( SyslogParam param ) {
		model = new SyslogModel( param );
		SyslogTable table = new SyslogTable( model, param );
		table.addMouseListener( new MouseAdapter() {
			public void mouseClicked( MouseEvent e ) {
				if ( e.getClickCount() == 2 ) {
					openDesc();
				}
			}
		});
		table.addKeyListener( new KeyAdapter() {
			public void keyPressed( KeyEvent e ) {
				if ( e.getKeyChar() == '\n' ) {
					openDesc();
					e.consume();
				}
			}
		});

		sp = new JScrollPane( table );
		sp.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
		add( sp, BorderLayout.CENTER );
		return table;
	}

	protected StatusBar createStatusBar() {
		StatusBar stat = new StatusBar( 4 );
		stat.setFont( param.getFont() );
		stat.setWidth( 0, 100 );
		stat.setWidth( 1, 100 );
		stat.setWidth( 2, 120 );
		stat.setHorizontalAlignment( 0, SwingConstants.RIGHT );
		stat.setHorizontalAlignment( 1, SwingConstants.RIGHT );
		stat.setHorizontalAlignment( 2, SwingConstants.LEFT );
		stat.setText( 0, "0 received." );
		stat.setText( 1, "0 saved." );

		String text;
		text = res.getString( "status.tip1" );
		stat.getObject( 0 ).setToolTipText( text );
		text = res.getString( "status.tip2" );
		stat.getObject( 1 ).setToolTipText( text );
		text = res.getString( "status.tip3" );
		stat.getObject( 2 ).setToolTipText( text );
		add( stat, BorderLayout.SOUTH );
		return stat;
	}

	DecimalFormat df1 = new DecimalFormat( "#,###" );
	SimpleDateFormat sdf1 = new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss" );
	public void syslogReceived( SyslogEvent ev ) {
		nRecv++;
		vecLog.add( ev );
		model.add( 0, ev );
		if ( param.getLimitLogSize() && vecLog.size() > param.getMaxLogSize()) {
			SyslogEvent old = (SyslogEvent )vecLog.remove( 0 );
			model.remove( old );
		}
		model.fireTableDataChanged();
		Date now = new Date();
		status.setText( 0, df1.format( nRecv ) + " received." );
		status.setText( 1, df1.format( vecLog.size() ) + " saved." );
		status.setText( 2, sdf1.format( now ));
		stat.update();
		if ( param.getPopup().isValid()) {
			java.util.List<IFilter> l = param.getPopup().listFilters();
			boolean bFilter = FilterUtility.isFiltered( l, ev );
			if ( !bFilter ) {
				try {
					param.getPopup().action( ev );
				} catch ( Exception ex ) {
					logger.error( "error", ex );
				}
			}
		}
	}

	// 名前を付けて保存
	public boolean writeFile( ActionEvent e ) {
		if ( vecLog.size() == 0 ) {
			return false;
		}
		String msg;
		String title;
		JFileChooser fc = new JFileChooser( new File( "." ).getAbsolutePath() ){
			private static final long serialVersionUID = 1L;

			public void approveSelection() {
				if ( getSelectedFile().exists() ) {
					String msg = res.getString( "write.message" );
					String title = res.getString( "write.title" );
					int nRet = JOptionPane.showConfirmDialog( this,
						msg, title,
						JOptionPane.YES_NO_OPTION );
					if ( nRet != JOptionPane.YES_OPTION ) {
						return;
					}
				}
				super.approveSelection();
			}
		};
		int nRet = fc.showSaveDialog( this );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return false;
		}
		PrintWriter out = null;
		try {
			OutputStream os = new FileOutputStream( fc.getSelectedFile() );
			OutputStreamWriter osw = new OutputStreamWriter( os );
			out = new PrintWriter( osw, true );
			SyslogEvent	ev;
			SyslogMsg	sysmsg;
			for ( int nLoop = 0; nLoop < vecLog.size(); nLoop++ ) {
				ev = (SyslogEvent )vecLog.get( nLoop );
				sysmsg = ev.getMessage();
				out.println( sysmsg.getData() );
			}
			os.flush();
			os.close();
			msg = res.getString( "write.end.message" );
			title = res.getString( "write.end.title" );
			JOptionPane.showMessageDialog( this,
					msg, title,
					JOptionPane.INFORMATION_MESSAGE );
		} catch ( IOException ex ) {
			logger.error( "write error", ex );
			msg = res.getString( "write.err1.message" );
			title = res.getString( "write.err1.title" );
			JOptionPane.showMessageDialog( this,
					msg + "\n" + ex.getMessage(), title,
					JOptionPane.ERROR_MESSAGE );
			return false;
		} finally {
			if ( out != null ) {
				out.close();
			}
		}
		return true;
	}

	// 全消去
	public void clear( ActionEvent e ) {
		if ( vecLog.size() == 0 ) {
			return;
		}
		String msg = res.getString( "clear.message" );
		String title = res.getString( "clear.title" );
		int nRet = JOptionPane.showConfirmDialog( this, msg, title,
			JOptionPane.YES_NO_CANCEL_OPTION );
		if ( nRet != JOptionPane.YES_OPTION && nRet != JOptionPane.NO_OPTION ) {
			return;
		}
		if ( nRet == JOptionPane.YES_OPTION ) {
			boolean bRet = writeFile( e );
			if ( !bRet ) {
				return;
			}
		}
		desc.setVisible( false );
		vecLog.clear();
		model.clear();
		model.fireTableDataChanged();
		status.setText( 1, "0 saved." );
		stat.update();
	}

	// 設定
	public void setOption( ActionEvent e ) {
		WindowUtility.center( this, setup );
		setup.setVisible( true );
	}

	// 設定の保存
	public void saveOption( ActionEvent e ) {
		String[] ext = { "xml" };
		String desc = res.getString( "save.desc" );
		ExtFileFilter ff = new ExtFileFilter( desc, ext );
		JFileChooser fc = new JFileChooser( new File( "." ).getAbsolutePath() ){
			private static final long serialVersionUID = 1L;

			public void approveSelection() {
				File sel = getSelectedFile();
				if ( getFileFilter() instanceof ExtFileFilter ) {
					if ( !sel.getPath().toLowerCase().endsWith( ".xml" )) {
						sel = new File( sel.getPath() + ".xml" );
						setSelectedFile( sel );
					}
				}
				if ( sel.exists() ) {
					String msg = res.getString( "save.message" );
					String title = res.getString( "save.title" );
					int nRet = JOptionPane.showConfirmDialog( this,
						msg, title,
						JOptionPane.YES_NO_OPTION );
					if ( nRet != JOptionPane.YES_OPTION ) {
						return;
					}
				}
				super.approveSelection();
			}
		};
		fc.setFileFilter( ff );
		int nRet = fc.showSaveDialog( this );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return;
		}

		String msg, title;
		try {
			DaemonManager manager = DaemonManager.getInstance();
			DOMConfiguration config = DOMConfiguration.createDocument( "Domain" );
			manager.saveConfig( config );

			int[]	nColWidth = new int[param.getColumnCount()];
			int[]	nOrder = new int[param.getColumnCount()];
			for ( int i = 0; i < param.getColumnCount(); i++ ) {
				nColWidth[i] = table.getColumnModel().getColumn(i).getWidth();
				nOrder[i] = table.convertColumnIndexToModel(i);
			}
			param.setColWidth( nColWidth );
			param.setOrder( nOrder );
			Component parent = SwingUtilities.getRoot( this );
			Rectangle rect = parent.getBounds( null );
			param.setBounds( rect );
			param.setRowHeight( table.getRowHeight() );

			IConfiguration c = config.addChild( "Viewer" );
			param.saveConfig( c );

			Writer out = new OutputStreamWriter(
					new FileOutputStream( fc.getSelectedFile() ), "UTF-8" );
			DOMWriter writer = new DOMWriter( out );
			writer.write( config, "Domain" );
			out.flush();
			out.close();

			msg = res.getString( "save.end.message" );
			title = res.getString( "save.end.title" );
			JOptionPane.showMessageDialog( this,
					msg, title,
					JOptionPane.INFORMATION_MESSAGE );
		} catch ( Exception ex ) {
			logger.error( "save error", ex );
			msg = res.getString( "save.err1.message" );
			title = res.getString( "save.err1.title" );
			JOptionPane.showMessageDialog( this,
					msg + "\n" + ex.getMessage(), title,
					JOptionPane.ERROR_MESSAGE );
		}
	}

	// 順序変更
	public void setOrder( ActionEvent e, boolean ord ) {
		model.setOrder( ord );
		int nSel = table.getSelectedRow();
		if ( table.getRowCount() > 0 && nSel >= 0 ) {
			nSel = table.getRowCount() - nSel - 1;
			selectRow( nSel );
		}
		model.fireTableDataChanged();
	}

	// フォント設定
	public void setFont( ActionEvent e ) {
		WindowUtility.center( this, font );
		font.setSelectedFont( getFont() );
		Font f = font.showDialog( this );
		if ( f != null ) {
			param.setFont( f );
			setFont( f );
			table.setFont( f );
		}
	}

	// Look & Feel設定
	public void setLF( ActionEvent e ) {
		String msg, title;
		String cls = e.getActionCommand();
		try {
			UIManager.setLookAndFeel( cls );
			SwingUtilities.updateComponentTreeUI( parent );
			SwingUtilities.updateComponentTreeUI( about );
			SwingUtilities.updateComponentTreeUI( setup );
			SwingUtilities.updateComponentTreeUI( search );
			SwingUtilities.updateComponentTreeUI( font );
			SwingUtilities.updateComponentTreeUI( desc );
			SwingUtilities.updateComponentTreeUI( stat );
			about.pack();
		} catch ( Exception ex ) {
			logger.error( "setLF error", ex );
			msg = res.getString( "lf.err1.message" );
			title = res.getString( "lf.err1.title" );
			JOptionPane.showMessageDialog( this,
					msg + ex, title,
					JOptionPane.ERROR_MESSAGE );
		}
	}

	protected DescDialog createDescDialog( SyslogParam param ) {
		DescDialog dlg;
		if ( getParent() instanceof Dialog ) {
			dlg = new DescDialog( (JDialog )getParent(), param );
		} else if ( getParent() instanceof JFrame ) {
			dlg = new DescDialog( (JFrame )getParent(), param );
		} else {
			dlg = new DescDialog( param );
		}

		dlg.BTN_PREV.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				prevDesc();
			}
		});
		dlg.BTN_NEXT.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				nextDesc();
			}
		});

		return dlg;
	}

	public void openDesc() {
		int		nPos = table.getSelectedRow();
		if ( table.getRowCount() > 0 && nPos >= 0 ) {
			table.setRowSelectionInterval( nPos, nPos );
			SyslogEvent ev = (SyslogEvent )model.getData( nPos );
			SyslogMsg msg = ev.getMessage();
			desc.setData( msg, model.getRowCount(), nPos );
			WindowUtility.center( this, desc );
			desc.setVisible( true );
		}
	}

	private void selectRow( int row ) {
		if ( row < 0 ) {
			table.clearSelection();
		} else {
			table.setRowSelectionInterval( row, row );
			Rectangle rect = table.getCellRect( row, 0, true );
			table.scrollRectToVisible( rect );
		}
	}

	public void prevDesc() {
		int		nPos = table.getSelectedRow();
		nPos--;
		selectRow( nPos );
		openDesc();
	}
	public void nextDesc() {
		int		nPos = table.getSelectedRow();
		nPos++;
		selectRow( nPos );
		openDesc();
	}

	protected StatisticsDialog createStatisticsDialog( SyslogParam param ) {
		StatisticsDialog dlg;
		if ( getParent() instanceof JDialog ) {
			dlg = new StatisticsDialog( (JDialog )getParent(), param, vecLog, table );
		} else if ( getParent() instanceof JFrame ) {
			dlg = new StatisticsDialog( (JFrame )getParent(), param, vecLog, table );
		} else {
			dlg = new StatisticsDialog( param, vecLog, table );
		}

		return dlg;
	}

	protected SearchDialog createSearchDialog( SyslogParam param ) {
		SearchDialog dlg;
		if ( getParent() instanceof Dialog ) {
			dlg = new SearchDialog( (JDialog )getParent(), param );
		} else if ( getParent() instanceof JFrame ) {
			dlg = new SearchDialog( (JFrame )getParent(), param );
		} else {
			dlg = new SearchDialog( param );
		}
		dlg.BTN_PREV.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				searchPrev();
			}
		});
		dlg.BTN_NEXT.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				searchNext();
			}
		});

		return dlg;
	}

	public void searchPrev() {
		int nPos = table.getSelectedRow();
		if ( nPos < 0 ) {
			nPos = table.getRowCount();
		}
		boolean bRet;
		bRet = searchPrev( nPos );
		if ( !bRet ) {
			String err = res.getString( "search.prev.error" );
			String msg = res.getString( "search.prev.message" );
			String title = res.getString( "search.prev.title" );
			int nRet = JOptionPane.showConfirmDialog( search,
						err + msg, title,
						JOptionPane.YES_NO_OPTION );
			if ( nRet != JOptionPane.YES_OPTION ) {
				return;
			}
			bRet = searchPrev( table.getRowCount() );
			if ( !bRet ) {
				JOptionPane.showMessageDialog( search,
						err, title,
						JOptionPane.WARNING_MESSAGE );
			}
		}
	}

	protected boolean searchPrev( int nPos ) {
		if ( table.getRowCount() > 0 ) {
			for ( int i = nPos - 1; i >= 0; i-- ) {
				SyslogEvent ev = (SyslogEvent )model.getData( i );
				if ( search.match( ev )) {
					selectRow( i );
					return true;
				}
			}
		}
		return false;
	}

	public void searchNext() {
		int nPos = table.getSelectedRow();
		if ( nPos < 0 ) {
			nPos = -1;
		}
		boolean bRet;
		bRet = searchNext( nPos );
		if ( !bRet ) {
			String err = res.getString( "search.next.error" );
			String msg = res.getString( "search.next.message" );
			String title = res.getString( "search.next.title" );
			int nRet = JOptionPane.showConfirmDialog( search,
						err + msg, title,
						JOptionPane.YES_NO_OPTION );
			if ( nRet != JOptionPane.YES_OPTION ) {
				return;
			}
			bRet = searchNext( -1 );
			if ( !bRet ) {
				JOptionPane.showMessageDialog( search,
						err, title,
						JOptionPane.WARNING_MESSAGE );
			}
		}
	}

	protected boolean searchNext( int nPos ) {
		if ( table.getRowCount() > 0 ) {
			for ( int i = nPos + 1; i < table.getRowCount(); i++ ) {
				SyslogEvent ev = (SyslogEvent )model.getData( i );
				if ( search.match( ev )) {
					selectRow( i );
					return true;
				}
			}
		}
		return false;
	}

	protected SetupDialog createSetupDialog( SyslogParam param ) {
		SetupDialog dlg;
		if ( getParent() instanceof JDialog ) {
			dlg = new SetupDialog( (JDialog )getParent(), param );
		} else if ( getParent() instanceof JFrame ) {
			dlg = new SetupDialog( (JFrame )getParent(), param );
		} else {
			dlg = new SetupDialog( param );
		}
		dlg.BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onSetupOk();
			}
		});

		return dlg;
	}

	public void onSetupOk() {
		setup.onOk();
		if ( setup.isColumnChange() ) {
			model.fireTableStructureChanged();
			table.setIconColumn();
			model.fireTableDataChanged();
		}
		if ( setup.isFilterChange()) {
			model.clear();
			for ( int i = 0; i < vecLog.size(); i++ ) {
				SyslogEvent ev = (SyslogEvent )vecLog.get( i );
				model.add( 0, ev );
			}
			model.fireTableDataChanged();
		}
		if ( param.getLimitLogSize() ) {
			for ( ;; ) {
				if ( vecLog.size() <= param.getMaxLogSize()) {
					break;
				}
				SyslogEvent old = (SyslogEvent )vecLog.remove( 0 );
				model.remove( old );
			}
			model.fireTableDataChanged();
		}
		repaint();
	}

	public void search( ActionEvent e ) {
		WindowUtility.center( this, search );
		search.setVisible( true );
	}

	public void statistics( ActionEvent e ) {
		WindowUtility.center( this, stat );
		stat.setVisible( true );
	}

	public void showHelp( ActionEvent e ) {
		WindowUtility.center( this, about );
		about.setVisible( true );
	}

}
