package	jp.co.sds_corp.syslog.viewer.statistics;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.syslog.viewer.SyslogParam;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* Graphの設定を行うダイアログです。
*/
public class GraphSetupDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	SyslogParam	param;
	JRadioButton	RD_INT1;
	JRadioButton	RD_INT2;
	JRadioButton	RD_INT3;
	JRadioButton	RD_INT4;
	JRadioButton	RD_TERM1;
	JRadioButton	RD_TERM2;
	JRadioButton	RD_TERM3;
	JRadioButton	RD_TERM4;
	JRadioButton	RD_GRP1;
	JRadioButton	RD_GRP2;
	JCheckBox		CHK_AUTO;
	JButton		BTN_OK = new JButton( "OK" );
	JButton		BTN_CANCEL = new JButton( "Cancel" );
	boolean		bOk;
	public static final int	INT_5MIN	= 1;
	public static final int	INT_15MIN	= 2;
	public static final int	INT_30MIN	= 3;
	public static final int	INT_1HOUR	= 4;
	int	interval;	// interval
	public static final int	TERM_6HOUR	= 1;
	public static final int	TERM_12HOUR	= 2;
	public static final int	TERM_24HOUR	= 3;
	public static final int	TERM_48HOUR	= 4;
	int	term;	// term
	public static final int	GRP_PRIORITY	= 1;
	public static final int	GRP_FACILITY	= 2;
	int	group;	// group
	boolean	autoUpdate;	// group
	ResourceBundle res;

	public GraphSetupDialog( JFrame parent, SyslogParam param ) {
		super( parent );
		this.param = param;

		createControls();
	}

	public GraphSetupDialog( JDialog parent, SyslogParam param ) {
		super( parent );
		this.param = param;

		createControls();
	}

	protected void createControls() {
		setModal( true );
		setResizable( false );

		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		text = res.getString( "statistics.setup.title" );
		setTitle( text );
		setSize( 300, 300 );
		setFont( param.getDefaultFont() );
		getContentPane().setLayout( new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ));
		WindowUtility.setHideOnEscape( this );

		getContentPane().add( createIntervalPanel() );
		getContentPane().add( createTermPanel() );
		getContentPane().add( createGroupPanel() );
		getContentPane().add( createAutoPanel() );

		// getContentPane().add( Box.createVerticalStrut( 50 ));

		Box buttonBox = new Box( BoxLayout.X_AXIS );
		buttonBox.add( BTN_OK );
		buttonBox.add( BTN_CANCEL );
		getContentPane().add( buttonBox );

		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onCancel();
			}
		});

	}

	private JPanel createIntervalPanel() {
		ButtonGroup grp;
		String text;

		JPanel panel = new JPanel();
		text = res.getString( "statistics.setup.tab1.title" );
		panel.setBorder( new TitledBorder( text ));
		grp = new ButtonGroup();
		text = res.getString( "statistics.setup.tab1.radio1" );
		RD_INT1 = new JRadioButton( text );
		grp.add( RD_INT1 );
		panel.add( RD_INT1 );
		text = res.getString( "statistics.setup.tab1.radio2" );
		RD_INT2 = new JRadioButton( text );
		grp.add( RD_INT2 );
		panel.add( RD_INT2 );
		text = res.getString( "statistics.setup.tab1.radio3" );
		RD_INT3 = new JRadioButton( text );
		grp.add( RD_INT3 );
		panel.add( RD_INT3 );
		text = res.getString( "statistics.setup.tab1.radio4" );
		RD_INT4 = new JRadioButton( text );
		grp.add( RD_INT4 );
		panel.add( RD_INT4 );

		interval = INT_5MIN;
		return panel;
	}

	private JPanel createTermPanel() {
		ButtonGroup grp;
		String	text;

		JPanel panel = new JPanel();
		text = res.getString( "statistics.setup.tab2.title" );
		panel.setBorder( new TitledBorder( text ));
		grp = new ButtonGroup();
		text = res.getString( "statistics.setup.tab2.radio1" );
		RD_TERM1 = new JRadioButton( text );
		grp.add( RD_TERM1 );
		panel.add( RD_TERM1 );
		text = res.getString( "statistics.setup.tab2.radio2" );
		RD_TERM2 = new JRadioButton( text );
		grp.add( RD_TERM2 );
		panel.add( RD_TERM2 );
		text = res.getString( "statistics.setup.tab2.radio3" );
		RD_TERM3 = new JRadioButton( text );
		grp.add( RD_TERM3 );
		panel.add( RD_TERM3 );
		text = res.getString( "statistics.setup.tab2.radio4" );
		RD_TERM4 = new JRadioButton( text );
		grp.add( RD_TERM4 );
		panel.add( RD_TERM4 );

		term = TERM_12HOUR;
		return panel;
	}

	private JPanel createGroupPanel() {
		ButtonGroup grp;
		String	text;

		JPanel panel = new JPanel();
		text = res.getString( "statistics.setup.tab3.title" );
		panel.setBorder( new TitledBorder( text ));
		grp = new ButtonGroup();
		text = res.getString( "statistics.setup.tab3.radio1" );
		RD_GRP1 = new JRadioButton( text );
		grp.add( RD_GRP1 );
		panel.add( RD_GRP1 );
		text = res.getString( "statistics.setup.tab3.radio2" );
		RD_GRP2 = new JRadioButton( text );
		grp.add( RD_GRP2 );
		panel.add( RD_GRP2 );

		group = GRP_PRIORITY;
		return panel;
	}

	private JPanel createAutoPanel() {
		String	text;
		JPanel panel = new JPanel();
		panel.setLayout( new BorderLayout() );
		text = res.getString( "statistics.setup.tab4.title" );
		panel.setBorder( new TitledBorder( text ));
		text = res.getString( "statistics.setup.tab4.text" );
		CHK_AUTO = new JCheckBox( text );
		panel.add( CHK_AUTO, BorderLayout.WEST );

		autoUpdate = false;
		return panel;
	}

	public void setVisible( boolean var ) {
		if ( var ) {
			switch ( interval ) {
			case INT_5MIN: RD_INT1.setSelected( true ); break;
			case INT_15MIN: RD_INT2.setSelected( true ); break;
			case INT_30MIN: RD_INT3.setSelected( true ); break;
			case INT_1HOUR: RD_INT4.setSelected( true ); break;
			}
			switch ( term ) {
			case TERM_6HOUR: RD_TERM1.setSelected( true ); break;
			case TERM_12HOUR: RD_TERM2.setSelected( true ); break;
			case TERM_24HOUR: RD_TERM3.setSelected( true ); break;
			case TERM_48HOUR: RD_TERM4.setSelected( true ); break;
			}
			switch ( group ) {
			case GRP_PRIORITY: RD_GRP1.setSelected( true ); break;
			case GRP_FACILITY: RD_GRP2.setSelected( true ); break;
			}
			CHK_AUTO.setSelected( autoUpdate );
		}
		super.setVisible( var );
	}

	public void onOk() {
		if ( RD_INT1.isSelected()) {
			interval = INT_5MIN;
		} else if ( RD_INT2.isSelected()) {
			interval = INT_15MIN;
		} else if ( RD_INT3.isSelected()) {
			interval = INT_30MIN;
		} else if ( RD_INT4.isSelected()) {
			interval = INT_1HOUR;
		}
		if ( RD_TERM1.isSelected()) {
			term = TERM_6HOUR;
		} else if ( RD_TERM2.isSelected()) {
			term = TERM_12HOUR;
		} else if ( RD_TERM3.isSelected()) {
			term = TERM_24HOUR;
		} else if ( RD_TERM4.isSelected()) {
			term = TERM_48HOUR;
		}
		if ( RD_GRP1.isSelected()) {
			group = GRP_PRIORITY;
		} else if ( RD_GRP2.isSelected()) {
			group = GRP_FACILITY;
		}
		autoUpdate = CHK_AUTO.isSelected();
		setVisible( false );
	}

	public void onCancel() {
		setVisible( false );
	}

	public boolean isOk() {
		return bOk;
	}

	/**
	* intervalを設定します。
	* @param	var		設定するデータ
	*/
	public void setInterval( int var ) {
		this.interval = var;
	}
	/**
	* intervalを取得します。
	* @return	interval
	*/
	public int getInterval() {
		return interval;
	}

	/**
	* groupを設定します。
	* @param	var		設定するデータ
	*/
	public void setGroup( int var ) {
		this.group = var;
	}
	/**
	* groupを取得します。
	* @return	group
	*/
	public int getGroup() {
		return group;
	}

	/**
	* termを設定します。
	* @param	var		設定するデータ
	*/
	public void setTerm( int var ) {
		this.term = var;
	}
	/**
	* termを取得します。
	* @return	term
	*/
	public int getTerm() {
		return term;
	}

	/**
	* autoUpdateを設定します。
	* @param	var		設定するデータ
	*/
	public void setAutoUpdate( boolean var ) {
		this.autoUpdate = var;
	}
	/**
	* autoUpdateを取得します。
	* @return	autoUpdate
	*/
	public boolean getAutoUpdate() {
		return autoUpdate;
	}

}
