package	jp.co.sds_corp.syslog.filter;

import java.io.Serializable;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;

/**
* syslogのフィルタリングを行なうクラスのためのインターフェースです。
*/
public interface IFilter extends IConfigurable, IConfigSavable, Serializable
{
	/**
	* syslogのがこのフィルタの条件に合致するかを検査します。
	* @param	e		データ受信イベント
	* @return	syslogのがこのフィルタの条件に合致すればtrue、しなければfalse
	*/
	public boolean match( SyslogEvent e );

	/**
	* このフィルタの条件が排他であるかどうかを返します。
	* @return	このフィルタの条件が排他であるかどうか
	*/
	public boolean isExclude();

	/**
	* このフィルタの排他条件を設定します。
	* @param	var		排他条件
	*/
	public void setExclude( boolean var );

	/**
	* このフィルタが有効であるかどうかを返します。
	* @return	このフィルタが有効であるかどうか
	*/
	public boolean isValid();

	/**
	* このフィルタの有効であるかどうかを設定します。
	* @param	var		有効フラグ
	*/
	public void setValid( boolean var );

	/**
	* 設定内容の文字列表現を取得します。
	* @return	設定内容の文字列表現
	*/
	public String paramString();

}
